@@ -4,38 +4,41 @@ Provides data marshalling to and from the DBus wire format
 @author: Tom Cocagne
 """
 
-import struct
+import codecs
 import re
+import struct
+
+import six
 
 from txdbus.error import MarshallingError
 
 
 invalid_obj_path_re = re.compile('[^a-zA-Z0-9_/]')
-if_re               = re.compile('[^A-Za-z0-9_.]')
-bus_re              = re.compile('[^A-Za-z0-9_.\-:]')
-mbr_re              = re.compile('[^A-Za-z0-9_]')
-dot_digit_re        = re.compile('\.\d')
+if_re = re.compile('[^A-Za-z0-9_.]')
+bus_re = re.compile('[^A-Za-z0-9_.\-:]')
+mbr_re = re.compile('[^A-Za-z0-9_]')
+dot_digit_re = re.compile('\.\d')
 
 
 #                Name      Type code   Alignment
-dbus_types = [ ('BYTE',        'y',     1),
-               ('BOOLEAN',     'b',     4),
-               ('INT16',       'n',     2),
-               ('UINT16',      'q',     2),
-               ('INT32',       'i',     4),
-               ('UINT32',      'u',     4),
-               ('INT64',       'x',     8),
-               ('UINT64',      't',     8),
-               ('DOUBLE',      'd',     8),
-               ('STRING',      's',     4), # (4-byte align for length)
-               ('OBJECT_PATH', 'o',     4), # (4-byte align for length)
-               ('SIGNATURE',   'g',     1),
-               ('ARRAY',       'a',     4), # (4-byte align for length)
-               ('STRUCT',      '(',     8),
-               ('VARIANT',     'v',     1), # (1-byte align for signature)
-               ('DICT_ENTRY',  '{',     8),
-               ('UNIX_FD',     'h',     4)
-               ]
+dbus_types = [('BYTE', 'y', 1),
+              ('BOOLEAN', 'b', 4),
+              ('INT16', 'n', 2),
+              ('UINT16', 'q', 2),
+              ('INT32', 'i', 4),
+              ('UINT32', 'u', 4),
+              ('INT64', 'x', 8),
+              ('UINT64', 't', 8),
+              ('DOUBLE', 'd', 8),
+              ('STRING', 's', 4),  # (4-byte align for length)
+              ('OBJECT_PATH', 'o', 4),  # (4-byte align for length)
+              ('SIGNATURE', 'g', 1),
+              ('ARRAY', 'a', 4),  # (4-byte align for length)
+              ('STRUCT', '(', 8),
+              ('VARIANT', 'v', 1),  # (1-byte align for signature)
+              ('DICT_ENTRY', '{', 8),
+              ('UNIX_FD', 'h', 4)
+              ]
 
 
 class Byte(int):
@@ -44,63 +47,72 @@ class Byte(int):
     encoded rather than the generic Python type
     """
     dbusSignature = 'y'
-    
+
+
 class Boolean(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'b'
-    
+
+
 class Int16(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'n'
-    
+
+
 class UInt16(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'q'
-    
+
+
 class Int32(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'i'
-    
+
+
 class UInt32(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'u'
-    
+
+
 class Int64(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'x'
-    
+
+
 class UInt64(int):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 't'
-    
+
+
 class Signature (str):
     """
     Used during Variant serialization to ensure that this type is
     encoded rather than the generic Python type
     """
     dbusSignature = 'g'
-    
+
+
 class ObjectPath (str):
     """
     Used during Variant serialization to ensure that this type is
@@ -109,16 +121,19 @@ class ObjectPath (str):
     dbusSignature = 'o'
 
 
-variantClassMap = { 'y' : Byte,
-                    'b' : Boolean,
-                    'n' : Int16,
-                    'q' : UInt16,
-                    'i' : Int32,
-                    'u' : UInt32,
-                    'x' : Int64,
-                    't' : UInt64,
-                    'g' : Signature,
-                    'o' : ObjectPath }
+variantClassMap = {
+    'y': Byte,
+    'b': Boolean,
+    'n': Int16,
+    'q': UInt16,
+    'i': Int32,
+    'u': UInt32,
+    'x': Int64,
+    't': UInt64,
+    'g': Signature,
+    'o': ObjectPath,
+}
+
 
 def validateObjectPath(p):
     """
@@ -138,7 +153,7 @@ def validateObjectPath(p):
         raise MarshallingError('Invalid characters contained in object path')
 
 
-def validateInterfaceName( n ):
+def validateInterfaceName(n):
     """
     Verifies that the supplied name is a valid DBus Interface name. Throws
     an L{error.MarshallingError} if the format is invalid
@@ -147,7 +162,7 @@ def validateInterfaceName( n ):
     @param n: A DBus interface name
     """
     try:
-        if not '.' in n:
+        if '.' not in n:
             raise Exception('At least two components required')
         if '..' in n:
             raise Exception('".." not allowed in interface names')
@@ -158,21 +173,23 @@ def validateInterfaceName( n ):
         if n[0].isdigit():
             raise Exception('Names may not begin with a digit')
         if if_re.search(n):
-            raise Exception('Names contains a character outside the set [A-Za-z0-9_.]')
+            raise Exception(
+                'Names contains a character outside the set [A-Za-z0-9_.]')
         if dot_digit_re.search(n):
-            raise Exception('No components of an interface name may begin with a digit')
-    except Exception, e:
+            raise Exception(
+                'No components of an interface name may begin with a digit')
+    except Exception as e:
         raise MarshallingError('Invalid interface name "%s": %s' % (n, str(e)))
 
 
-def validateErrorName( n ):
+def validateErrorName(n):
     try:
-        validateInterfaceName( n )
-    except MarshallingError, e:
-        raise MarshallingError( str(e).replace( 'interface', 'error', 1 ) )
+        validateInterfaceName(n)
+    except MarshallingError as e:
+        raise MarshallingError(str(e).replace('interface', 'error', 1))
 
-    
-def validateBusName( n ):
+
+def validateBusName(n):
     """
     Verifies that the supplied name is a valid DBus Bus name. Throws
     an L{error.MarshallingError} if the format is invalid
@@ -181,7 +198,7 @@ def validateBusName( n ):
     @param n: A DBus bus name
     """
     try:
-        if not '.' in n:
+        if '.' not in n:
             raise Exception('At least two components required')
         if '..' in n:
             raise Exception('".." not allowed in bus names')
@@ -192,14 +209,16 @@ def validateBusName( n ):
         if n[0].isdigit():
             raise Exception('Names may not begin with a digit')
         if bus_re.search(n):
-            raise Exception('Names contains a character outside the set [A-Za-z0-9_.\-:]')
+            raise Exception(
+                'Names contains a character outside the set [A-Za-z0-9_.\-:]')
         if not n[0] == ':' and dot_digit_re.search(n):
-            raise Exception('No coponents of an interface name may begin with a digit')
-    except Exception, e:
+            raise Exception(
+                'No coponents of an interface name may begin with a digit')
+    except Exception as e:
         raise MarshallingError('Invalid bus name "%s": %s' % (n, str(e)))
 
-    
-def validateMemberName( n ):
+
+def validateMemberName(n):
     """
     Verifies that the supplied name is a valid DBus member name. Throws
     an L{error.MarshallingError} if the format is invalid
@@ -215,14 +234,15 @@ def validateMemberName( n ):
         if n[0].isdigit():
             raise Exception('Names may not begin with a digit')
         if mbr_re.search(n):
-            raise Exception('Names contains a character outside the set [A-Za-z0-9_]')
-    except Exception, e:
+            raise Exception(
+                'Names contains a character outside the set [A-Za-z0-9_]')
+    except Exception as e:
         raise MarshallingError('Invalid member name "%s": %s' % (n, str(e)))
 
-    
 
-# XXX: This could be made *much* smarter (handle objects and recursive structures)
-def sigFromPy( pobj ):
+# XXX: This could be made *much* smarter (handle objects and recursive
+# structures)
+def sigFromPy(pobj):
     """
     Returns the DBus signature type for the argument. If the argument is an
     instance of one of the type wrapper classes, the exact type signature
@@ -234,72 +254,90 @@ def sigFromPy( pobj ):
     @returns: The DBus signature for the supplied Python object
     """
     sig = getattr(pobj, 'dbusSignature', None)
-    
+
     if sig is not None:
         return sig
-    
-    elif isinstance(pobj,        int): return 'i'
-    elif isinstance(pobj,       long): return 'x'
-    elif isinstance(pobj,      float): return 'd'
-    elif isinstance(pobj, basestring): return 's'
-    
-    elif isinstance(pobj,       list):
+
+    elif isinstance(pobj, bool):
+        return 'b'
+    elif isinstance(pobj, int):
+        return 'i'
+    elif isinstance(pobj, six.integer_types):
+        return 'x'
+    elif isinstance(pobj, float):
+        return 'd'
+    elif isinstance(pobj, six.string_types):
+        return 's'
+    elif isinstance(pobj, bytearray):
+        return 'ay'
+
+    elif isinstance(pobj, list):
         vtype = type(pobj[0])
         same = True
         for v in pobj[1:]:
-            if not vtype is type(v):
+            if not isinstance(v, vtype):
                 same = False
         if same:
             return 'a' + sigFromPy(pobj[0])
         else:
             return 'av'
-    
-    elif isinstance(pobj,       dict):
-        values = pobj.values()
+
+    elif isinstance(pobj, tuple):
+        return '(' + ''.join(sigFromPy(e) for e in pobj) + ')'
+
+    elif isinstance(pobj, dict):
         same = True
-        vtype = type(values[0])
-        for v in values[1:]:
-            if not vtype is type(v):
+        vtype = None
+        for k, v in six.iteritems(pobj):
+            if vtype is None:
+                vtype = type(v)
+            elif not isinstance(v, vtype):
                 same = False
         if same:
-            return 'a{' + sigFromPy(pobj.keys()[0]) + sigFromPy(pobj.values()[0]) + '}'
+            return 'a{' + sigFromPy(k) + sigFromPy(v) + '}'
         else:
-            return 'a{' + sigFromPy(pobj.keys()[0]) + 'v}'
-    
-    else:
-        raise MarshallingError('Invalid Python type for variant: ' + repr(pobj))
+            return 'a{' + sigFromPy(k) + 'v}'
 
-    
-#-------------------------------------------------------------------------------
+    else:
+        raise MarshallingError(
+            'Invalid Python type for variant: ' +
+            repr(pobj))
+
+
+# ------------------------------------------------------------------------
 #                          Marshalling Functions
 # Padding:
 #    - All data types must be padded to the correct alignment
 #    - All padding bytes must be nul
 #
-padding = { 0 : '\0' * 0,
-            1 : '\0' * 1,
-            2 : '\0' * 2,
-            3 : '\0' * 3,
-            4 : '\0' * 4,
-            5 : '\0' * 5,
-            6 : '\0' * 6,
-            7 : '\0' * 7 }
+padding = {
+    0: b'\0' * 0,
+    1: b'\0' * 1,
+    2: b'\0' * 2,
+    3: b'\0' * 3,
+    4: b'\0' * 4,
+    5: b'\0' * 5,
+    6: b'\0' * 6,
+    7: b'\0' * 7,
+}
 
-def genpad( align ):
-    return lambda x : padding[ x % align and (align - x%align) or 0 ]
 
-pad = dict()
+def genpad(align):
+    return lambda x: padding[x % align and (align - x % align) or 0]
+
+
+pad = {}
 
 for name, tcode, align in dbus_types:
-    pad[ tcode ] = genpad(align)
+    pad[tcode] = genpad(align)
 
 pad['header'] = genpad(8)
 
 
-#-------------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 #                    Signature Generator/Iterator
 #
-def genCompleteTypes( compoundSig ):
+def genCompleteTypes(compoundSig):
     """
     Generator function used to iterate over each complete,
     top-level type contained in in a signature. Ex::
@@ -307,11 +345,10 @@ def genCompleteTypes( compoundSig ):
       "i(ii)i"    => [ 'i', '(ii)',    'i' ]
       "i(i(ii))i" => [ 'i', '(i(ii))', 'i' ]
     """
-    i     = 0
-    start = 0
-    end   = len(compoundSig)
+    i = 0
+    end = len(compoundSig)
 
-    def find_end( idx, b, e ):
+    def find_end(idx, b, e):
         depth = 1
         while idx < end:
             subc = compoundSig[idx]
@@ -322,34 +359,32 @@ def genCompleteTypes( compoundSig ):
                 if depth == 0:
                     return idx
             idx += 1
-    
+
     while i < end:
         c = compoundSig[i]
-        
+
         if c == '(':
-            x = find_end(i+1, '(', ')')
-            yield compoundSig[i:x+1]
+            x = find_end(i + 1, '(', ')')
+            yield compoundSig[i:x + 1]
             i = x
 
         elif c == '{':
-            x = find_end(i+1, '{', '}')
-            yield compoundSig[i:x+1]
+            x = find_end(i + 1, '{', '}')
+            yield compoundSig[i:x + 1]
             i = x
 
         elif c == 'a':
-            start = i
-            g = genCompleteTypes( compoundSig[i+1:] )
-            ct = g.next()
+            g = genCompleteTypes(compoundSig[i + 1:])
+            ct = six.next(g)
             i += len(ct)
             yield 'a' + ct
-            
+
         else:
             yield c
         i += 1
 
 
-
-#-------------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 #                          Marshalling Functions
 # General:
 #    - All values must be padded to proper alignment
@@ -382,32 +417,46 @@ def genCompleteTypes( compoundSig ):
 #
 #
 
-def marshal_byte( ct, var, start_byte, lendian ):
-    return 1, [ struct.pack( lendian and '<B' or '>B', var) ]
+def marshal_byte(ct, var, start_byte, lendian, oobFDs):
+    return 1, [struct.pack(lendian and '<B' or '>B', var)]
 
-def marshal_boolean( ct, var, start_byte, lendian ):
-    return 4, [ struct.pack( lendian and '<I' or '>I', 1 if var else 0) ]
 
-def marshal_int16( ct, var, start_byte, lendian ):
-    return 2, [ struct.pack( lendian and '<h' or '>h', var) ]
+def marshal_boolean(ct, var, start_byte, lendian, oobFDs):
+    return 4, [struct.pack(lendian and '<I' or '>I', 1 if var else 0)]
 
-def marshal_uint16( ct, var, start_byte, lendian ):
-    return 2, [ struct.pack( lendian and '<H' or '>H', var) ]
 
-def marshal_int32( ct, var, start_byte, lendian ):
-    return 4, [ struct.pack( lendian and '<i' or '>i', var) ]
+def marshal_int16(ct, var, start_byte, lendian, oobFDs):
+    return 2, [struct.pack(lendian and '<h' or '>h', var)]
 
-def marshal_uint32( ct, var, start_byte, lendian ):
-    return 4, [ struct.pack( lendian and '<I' or '>I', var) ]
 
-def marshal_int64( ct, var, start_byte, lendian ):
-    return 8, [ struct.pack( lendian and '<q' or '>q', var) ]
+def marshal_uint16(ct, var, start_byte, lendian, oobFDs):
+    return 2, [struct.pack(lendian and '<H' or '>H', var)]
 
-def marshal_uint64( ct, var, start_byte, lendian ):
-    return 8, [ struct.pack( lendian and '<Q' or '>Q', var) ]
 
-def marshal_double( ct, var, start_byte, lendian ):
-    return 8, [ struct.pack( lendian and '<d' or '>d', var) ]
+def marshal_int32(ct, var, start_byte, lendian, oobFDs):
+    return 4, [struct.pack(lendian and '<i' or '>i', var)]
+
+
+def marshal_uint32(ct, var, start_byte, lendian, oobFDs):
+    return 4, [struct.pack(lendian and '<I' or '>I', var)]
+
+
+def marshal_int64(ct, var, start_byte, lendian, oobFDs):
+    return 8, [struct.pack(lendian and '<q' or '>q', var)]
+
+
+def marshal_uint64(ct, var, start_byte, lendian, oobFDs):
+    return 8, [struct.pack(lendian and '<Q' or '>Q', var)]
+
+
+def marshal_double(ct, var, start_byte, lendian, oobFDs):
+    return 8, [struct.pack(lendian and '<d' or '>d', var)]
+
+
+def marshal_unix_fd(ct, var, start_byte, lendian, oobFDs):
+    index = len(oobFDs)
+    oobFDs.append(var)
+    return 4, [struct.pack(lendian and '<I' or '>I', index)]
 
 
 # STRING:
@@ -417,20 +466,24 @@ def marshal_double( ct, var, start_byte, lendian ):
 #       2 - string data (no embedded nuls)
 #       3 - terminating nul byte
 #
-def marshal_string( ct, var, start_byte, lendian ):
-    if not isinstance(var, basestring):
+def marshal_string(ct, var, start_byte, lendian, oobFDs):
+    if not isinstance(var, six.string_types):
         raise MarshallingError('Required string. Received: ' + repr(var))
     if var.find('\0') != -1:
-        raise MarshallingError('Embedded nul characters are not allowed within DBus strings')
-    return 4 + len(var) + 1, [ struct.pack( lendian and '<I' or '>I', len(var)), var, '\0' ]
+        raise MarshallingError(
+            'Embedded nul characters are not allowed within DBus strings')
+    var = codecs.encode(var, 'utf-8')
+    return 4 + \
+        len(var) + \
+        1, [struct.pack(lendian and '<I' or '>I', len(var)), var, b'\0']
 
 
 # OBJECT_PATH:
 #    - Identical to string
-#    
-def marshal_object_path( ct, var, start_byte, lendian ):
+#
+def marshal_object_path(ct, var, start_byte, lendian, oobFDs):
     validateObjectPath(var)
-    return marshal_string( ct, var, start_byte, lendian )
+    return marshal_string(ct, var, start_byte, lendian, oobFDs)
 
 
 # SIGNATURE:
@@ -441,9 +494,11 @@ def marshal_object_path( ct, var, start_byte, lendian ):
 #       1 - Single byte length
 #       2 - Valid signature string
 #       3 - terminating nul byte
-def marshal_signature( ct, var, start_byte, lendian ):
+def marshal_signature(ct, var, start_byte, lendian, oobFDs):
     # XXX validate signature
-    return 2 + len(var), [struct.pack(lendian and '<B' or '>B', len(var)), var, '\0']
+    var = codecs.encode(var, 'ascii')
+    return 2 + \
+        len(var), [struct.pack(lendian and '<B' or '>B', len(var)), var, b'\0']
 
 
 # ARRAY:
@@ -452,55 +507,58 @@ def marshal_signature( ct, var, start_byte, lendian ):
 #       1 - UINT32 length of array data (does not include alignment padding)
 #       2 - Padding to required alignment of contained data type
 #       3 - each array element
-def marshal_array( ct, var, start_byte, lendian ):
-    chunks   = list()
+def marshal_array(ct, var, start_byte, lendian, oobFDs):
+    chunks = []
     data_len = 0
-    tsig     = ct[1:]   # strip of leading 'a'
-    tcode    = tsig[0] # type of array element
+    tsig = ct[1:]   # strip of leading 'a'
+    tcode = tsig[0]  # type of array element
 
-    start_byte += 4 # for array size
-    
-    initial_padding = pad[tcode]( start_byte )
+    start_byte += 4  # for array size
+
+    initial_padding = pad[tcode](start_byte)
 
     if initial_padding:
         start_byte += len(initial_padding)
-        chunks.append( initial_padding )
+        chunks.append(initial_padding)
 
-    if isinstance(var, (list, tuple)):
+    if isinstance(var, (list, tuple, bytearray)):
         arr_list = var
     elif isinstance(var, dict):
-        arr_list = [ tpl for tpl in var.iteritems() ]
+        arr_list = [tpl for tpl in six.iteritems(var)]
     else:
-        raise MarshallingError('List, Tuple, or Dictionary required for DBus array. Received: ' + repr(var))
+        raise MarshallingError(
+            'List, Tuple, Bytearray, or Dictionary required for DBus array. '
+            ' Received: ' + repr(var)
+        )
 
     for item in arr_list:
 
-        padding = pad[tcode]( start_byte )
+        padding = pad[tcode](start_byte)
 
         if padding:
             start_byte += len(padding)
-            data_len   += len(padding)
-            chunks.append( padding )
-        
-        nbytes, vchunks = marshallers[ tcode ]( tsig, item, start_byte, lendian )
+            data_len += len(padding)
+            chunks.append(padding)
+
+        nbytes, vchunks = marshallers[tcode](
+            tsig, item, start_byte, lendian, oobFDs)
 
         start_byte += nbytes
-        data_len   += nbytes
-        
-        chunks.extend( vchunks )
-        
+        data_len += nbytes
 
-    chunks.insert(0, struct.pack( lendian and '<I' or '>I', data_len))
+        chunks.extend(vchunks)
+
+    chunks.insert(0, struct.pack(lendian and '<I' or '>I', data_len))
 
     return 4 + len(initial_padding) + data_len, chunks
-    
+
 
 # STRUCT:
 #    - Must start on 8 byte boundary
 #    - Content consists of each field marshaled in sequence
 #
-def marshal_struct( ct, var, start_byte, lendian ):
-    return marshal( ct[1:-1], var, start_byte, lendian )
+def marshal_struct(ct, var, start_byte, lendian, oobFDs):
+    return marshal(ct[1:-1], var, start_byte, lendian, oobFDs)
 
 
 marshal_dictionary = marshal_struct
@@ -512,58 +570,64 @@ marshal_dictionary = marshal_struct
 #       1 - Marshaled SIGNATURE
 #       2 - Any required padding to align the type specified in the signature
 #       3 - Marshaled value
-def marshal_variant( ct, var, start_byte, lendian ):
+def marshal_variant(ct, var, start_byte, lendian, oobFDs):
     # XXX: ensure only a single, complete type is in the siguature
     bstart = start_byte
-    
+
     vsig = sigFromPy(var)
-    
-    nbytes, chunks = marshal_signature( ct, sigFromPy(var), start_byte, lendian )
+
+    nbytes, chunks = marshal_signature(
+        ct, sigFromPy(var), start_byte, lendian, oobFDs)
 
     start_byte += nbytes
 
-    padding = pad[vsig[0]]( start_byte )
-        
+    padding = pad[vsig[0]](start_byte)
+
     if padding:
         start_byte += len(padding)
-        chunks.append( padding )
-        
-    rnbytes, rchunks = marshal( vsig, [var], start_byte, lendian )
+        chunks.append(padding)
+
+    rnbytes, rchunks = marshal(vsig, [var], start_byte, lendian)
 
     start_byte += rnbytes
-    chunks.extend( rchunks )
-    
+    chunks.extend(rchunks)
+
     return start_byte - bstart, chunks
 
 
-
-marshallers = { 'y' : marshal_byte,
-                'b' : marshal_boolean,
-                'n' : marshal_int16,
-                'q' : marshal_uint16,
-                'i' : marshal_int32,
-                'u' : marshal_uint32,
-                'x' : marshal_int64,
-                't' : marshal_uint64,
-                'd' : marshal_double,
-                's' : marshal_string,
-                'o' : marshal_object_path,
-                'g' : marshal_signature,
-                'a' : marshal_array,
-                '(' : marshal_struct,
-                'v' : marshal_variant,
-                '{' : marshal_dictionary,
-                'h' : marshal_uint32 }
+marshallers = {
+    'y': marshal_byte,
+    'b': marshal_boolean,
+    'n': marshal_int16,
+    'q': marshal_uint16,
+    'i': marshal_int32,
+    'u': marshal_uint32,
+    'x': marshal_int64,
+    't': marshal_uint64,
+    'd': marshal_double,
+    's': marshal_string,
+    'o': marshal_object_path,
+    'g': marshal_signature,
+    'a': marshal_array,
+    '(': marshal_struct,
+    'v': marshal_variant,
+    '{': marshal_dictionary,
+    'h': marshal_unix_fd,
+}
 
 
-def marshal( compoundSignature, variableList, startByte = 0, lendian=True ):
+def marshal(compoundSignature, variableList,
+            startByte=0, lendian=True, oobFDs=None):
     """
     Encodes the Python objects in variableList into the DBus wire-format
     matching the supplied compoundSignature. This function retuns a list of
     binary strings is rather than a single string to simplify the recursive
     marshalling algorithm. A single string may be easily obtained from the
     result via: ''.join(list_of_binary_strings)
-    
+
+    Any UNIX_FD 'h' type is encoded per spec and the respective FD appended
+    to oobFDs which should be supplied as an empty list.
+
     @type compoundSignature: C{string}
     @param compoundSignature: DBus signature specifying the types of the
                               variables to encode
@@ -582,68 +646,86 @@ def marshal( compoundSignature, variableList, startByte = 0, lendian=True ):
     @param lendian: True if the data should be serialized in
                     little-endian format
 
-    
+
     @returns: (number_of_encoded_bytes, list_of_binary_strings)
     """
-    chunks = list()
+    chunks = []
     bstart = startByte
 
     if hasattr(variableList, 'dbusOrder'):
         order = getattr(variableList, 'dbusOrder')
-        variableList = [ getattr(variableList, attr_name) for attr_name in order ]
+        variableList = [getattr(variableList, attr_name)
+                        for attr_name in order]
+
+    for ct, var in zip(genCompleteTypes(compoundSignature), variableList):
+        tcode = ct[0]
+        padding = pad[tcode](startByte)
 
-    for ct, var in zip(genCompleteTypes( compoundSignature ), variableList):
-        tcode   = ct[0]
-        padding = pad[tcode]( startByte )
-        
         if padding:
             startByte += len(padding)
-            chunks.append( padding )
-        
-        nbytes, vchunks = marshallers[ tcode ]( ct, var, startByte, lendian )
+            chunks.append(padding)
+
+        nbytes, vchunks = marshallers[tcode](
+            ct, var, startByte, lendian, oobFDs)
 
         startByte += nbytes
-        
-        chunks.extend( vchunks )
-        
+
+        chunks.extend(vchunks)
 
     return startByte - bstart, chunks
 
 
-
-#-------------------------------------------------------------------------------
-#-------------------------------------------------------------------------------
+# ------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 #                         Unmarshalling Functions
-#-------------------------------------------------------------------------------
-#-------------------------------------------------------------------------------
+# ------------------------------------------------------------------------
+# ------------------------------------------------------------------------
 
 
-def unmarshal_byte(ct, data, offset, lendian):
-    return 1, struct.unpack_from( lendian and '<B' or '>B', data, offset)[0]
+def unmarshal_byte(ct, data, offset, lendian, oobFDs):
+    return 1, struct.unpack_from(lendian and '<B' or '>B', data, offset)[0]
 
-def unmarshal_boolean(ct, data, offset, lendian):
-    return 4, struct.unpack_from( lendian and '<I' or '>I', data, offset)[0] != 0
 
-def unmarshal_int16(ct, data, offset, lendian):
-    return 2, struct.unpack_from( lendian and '<h' or '>h', data, offset)[0]
+def unmarshal_boolean(ct, data, offset, lendian, oobFDs):
+    return 4, struct.unpack_from(
+        lendian and '<I' or '>I', data, offset)[0] != 0
 
-def unmarshal_uint16(ct, data, offset, lendian):
-    return 2, struct.unpack_from( lendian and '<H' or '>H', data, offset)[0]
 
-def unmarshal_int32(ct, data, offset, lendian):
-    return 4, struct.unpack_from( lendian and '<i' or '>i', data, offset)[0]
+def unmarshal_int16(ct, data, offset, lendian, oobFDs):
+    return 2, struct.unpack_from(lendian and '<h' or '>h', data, offset)[0]
 
-def unmarshal_uint32(ct, data, offset, lendian):
-    return 4, struct.unpack_from( lendian and '<I' or '>I', data, offset)[0]
 
-def unmarshal_int64(ct, data, offset, lendian):
-    return 8, struct.unpack_from( lendian and '<q' or '>q', data, offset)[0]
+def unmarshal_uint16(ct, data, offset, lendian, oobFDs):
+    return 2, struct.unpack_from(lendian and '<H' or '>H', data, offset)[0]
 
-def unmarshal_uint64(ct, data, offset, lendian):
-    return 8, struct.unpack_from( lendian and '<Q' or '>Q', data, offset)[0]
 
-def unmarshal_double(ct, data, offset, lendian):
-    return 8, struct.unpack_from( lendian and '<d' or '>d', data, offset)[0]
+def unmarshal_int32(ct, data, offset, lendian, oobFDs):
+    return 4, struct.unpack_from(lendian and '<i' or '>i', data, offset)[0]
+
+
+def unmarshal_uint32(ct, data, offset, lendian, oobFDs):
+    return 4, struct.unpack_from(lendian and '<I' or '>I', data, offset)[0]
+
+
+def unmarshal_int64(ct, data, offset, lendian, oobFDs):
+    return 8, struct.unpack_from(lendian and '<q' or '>q', data, offset)[0]
+
+
+def unmarshal_uint64(ct, data, offset, lendian, oobFDs):
+    return 8, struct.unpack_from(lendian and '<Q' or '>Q', data, offset)[0]
+
+
+def unmarshal_double(ct, data, offset, lendian, oobFDs):
+    return 8, struct.unpack_from(lendian and '<d' or '>d', data, offset)[0]
+
+
+def unmarshal_unix_fd(ct, data, offset, lendian, oobFDs):
+    index = struct.unpack_from(lendian and '<I' or '>I', data, offset)[0]
+    try:
+        fd = oobFDs[index]
+    except IndexError:
+        fd = None
+    return 4, fd
 
 
 # STRING:
@@ -653,14 +735,15 @@ def unmarshal_double(ct, data, offset, lendian):
 #       2 - string data (no embedded nuls)
 #       3 - terminating nul byte
 #
-def unmarshal_string(ct, data, offset, lendian):
-    slen = struct.unpack_from( lendian and '<I' or '>I', data, offset)[0]
-    return 4 + slen + 1, data[ offset + 4 :  offset + 4 + slen ]
-    
+def unmarshal_string(ct, data, offset, lendian, oobFDs):
+    slen = struct.unpack_from(lendian and '<I' or '>I', data, offset)[0]
+    s = codecs.decode(data[offset + 4: offset + 4 + slen], 'utf-8')
+    return 4 + slen + 1, s
+
 
 # OBJECT_PATH:
 #    - Identical to string
-#    
+#
 unmarshal_object_path = unmarshal_string
 
 
@@ -672,10 +755,11 @@ unmarshal_object_path = unmarshal_string
 #       1 - Single byte length
 #       2 - Valid signature string
 #       3 - terminating nul byte
-def unmarshal_signature(ct, data, offset, lendian):
-    slen = struct.unpack_from( lendian and '<B' or '>B', data, offset)[0]
-    return 1 + slen + 1, data[ offset + 1 : offset + 1 + slen ]
-    
+def unmarshal_signature(ct, data, offset, lendian, oobFDs):
+    slen = struct.unpack_from(lendian and '<B' or '>B', data, offset)[0]
+    s = codecs.decode(data[offset + 1: offset + 1 + slen], 'ascii')
+    return 1 + slen + 1, s
+
 
 # ARRAY:
 #    - Max length is 2^26
@@ -683,45 +767,46 @@ def unmarshal_signature(ct, data, offset, lendian):
 #       1 - UINT32 length of array data (does not include alignment padding)
 #       2 - Padding to required alignment of contained data type
 #       3 - each array element
-def unmarshal_array(ct, data, offset, lendian):
+def unmarshal_array(ct, data, offset, lendian, oobFDs):
     start_offset = offset
-    values       = list()
-    data_len     = struct.unpack_from( lendian and '<I' or '>I', data, offset)[0]
-    tsig         = ct[1:]  # strip of leading 'a'
-    tcode        = tsig[0] # type of array element
-    
+    values = []
+    data_len = struct.unpack_from(lendian and '<I' or '>I', data, offset)[0]
+    tsig = ct[1:]  # strip of leading 'a'
+    tcode = tsig[0]  # type of array element
+
     offset += 4                         # 4-byte data length
-    offset += len(pad[tcode]( offset )) # padding length
-    
+    offset += len(pad[tcode](offset))  # padding length
+
     end_offset = offset + data_len
-    
+
     while offset < end_offset:
 
         offset += len(pad[tcode](offset))
-        
-        nbytes, value = unmarshallers[ tcode ]( tsig, data, offset, lendian )
+
+        nbytes, value = unmarshallers[tcode](
+            tsig, data, offset, lendian, oobFDs)
 
         offset += nbytes
-        values.append( value )
+        values.append(value)
 
     if not offset == end_offset:
         raise MarshallingError('Invalid array encoding')
 
     if tcode == '{':
-        d = dict()
+        d = {}
         for item in values:
-            d[ item[0] ] = item[1]
+            d[item[0]] = item[1]
         values = d
 
     return offset - start_offset, values
-    
+
 
 # STRUCT:
 #    - Must start on 8 byte boundary
 #    - Content consists of each field marshaled in sequence
 #
-def unmarshal_struct(ct, data, offset, lendian):
-    return unmarshal( ct[1:-1], data, offset, lendian )
+def unmarshal_struct(ct, data, offset, lendian, oobFDs):
+    return unmarshal(ct[1:-1], data, offset, lendian, oobFDs)
 
 
 unmarshal_dictionary = unmarshal_struct
@@ -733,44 +818,44 @@ unmarshal_dictionary = unmarshal_struct
 #       1 - Marshaled SIGNATURE
 #       2 - Any required padding to align the type specified in the signature
 #       3 - Marshaled value
-def unmarshal_variant(ct, data, offset, lendian):
+def unmarshal_variant(ct, data, offset, lendian, oobFDs):
     # XXX: ensure only a single, complete type is in the siguature
     start_offset = offset
-    nsig, vsig = unmarshal_signature( ct, data, offset, lendian )
+    nsig, vsig = unmarshal_signature(ct, data, offset, lendian, oobFDs)
 
     offset += nsig
-    
+
     offset += len(pad[vsig[0]](offset))
 
-    nvar, value = unmarshal( vsig, data, offset, lendian )
+    nvar, value = unmarshal(vsig, data, offset, lendian, oobFDs)
 
     offset += nvar
-    
+
     return offset - start_offset, value[0]
 
 
+unmarshallers = {
+    'y': unmarshal_byte,
+    'b': unmarshal_boolean,
+    'n': unmarshal_int16,
+    'q': unmarshal_uint16,
+    'i': unmarshal_int32,
+    'u': unmarshal_uint32,
+    'x': unmarshal_int64,
+    't': unmarshal_uint64,
+    'd': unmarshal_double,
+    's': unmarshal_string,
+    'o': unmarshal_object_path,
+    'g': unmarshal_signature,
+    'a': unmarshal_array,
+    '(': unmarshal_struct,
+    'v': unmarshal_variant,
+    '{': unmarshal_dictionary,
+    'h': unmarshal_unix_fd,
+}
 
 
-unmarshallers = { 'y' : unmarshal_byte,
-                  'b' : unmarshal_boolean,
-                  'n' : unmarshal_int16,
-                  'q' : unmarshal_uint16,
-                  'i' : unmarshal_int32,
-                  'u' : unmarshal_uint32,
-                  'x' : unmarshal_int64,
-                  't' : unmarshal_uint64,
-                  'd' : unmarshal_double,
-                  's' : unmarshal_string,
-                  'o' : unmarshal_object_path,
-                  'g' : unmarshal_signature,
-                  'a' : unmarshal_array,
-                  '(' : unmarshal_struct,
-                  'v' : unmarshal_variant,
-                  '{' : unmarshal_dictionary,
-                  'h' : unmarshal_uint32 }
-
-
-def unmarshal( compoundSignature, data, offset = 0, lendian = True ):
+def unmarshal(compoundSignature, data, offset=0, lendian=True, oobFDs=None):
     """
     Unmarshals DBus encoded data.
 
@@ -786,23 +871,19 @@ def unmarshal( compoundSignature, data, offset = 0, lendian = True ):
 
     @type lendian: C{bool}
     @param lendian: True if data is encoded in little-endian format
-    
+
     @returns: (number_of_bytes_decoded, list_of_values)
     """
-    values       = list()
+    values = []
     start_offset = offset
 
-    for ct in genCompleteTypes( compoundSignature ):
-        tcode   = ct[0]
-        offset += len(pad[tcode]( offset ))
-                
-        nbytes, value = unmarshallers[ tcode ]( ct, data, offset, lendian )
+    for ct in genCompleteTypes(compoundSignature):
+        tcode = ct[0]
+        offset += len(pad[tcode](offset))
+
+        nbytes, value = unmarshallers[tcode](ct, data, offset, lendian, oobFDs)
 
         offset += nbytes
-        values.append( value )
-
+        values.append(value)
 
     return offset - start_offset, values
-
-
-
